@@ -2,7 +2,7 @@
    dynload.h : This file is part of pstoedit
    declarations for dynamic loading of drivers
 
-   Copyright (C) 1993 - 2019 Wolfgang Glunz, wglunz35_AT_pstoedit.net
+   Copyright (C) 1993 - 2021 Wolfgang Glunz, wglunz35_AT_pstoedit.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -77,7 +77,7 @@ static char *dlerror()
 {
 	LPVOID lpMsgBuf;
   //  LPVOID lpDisplayBuf;
-    DWORD dw = GetLastError(); 
+    const DWORD dw = GetLastError(); 
 
     (void)FormatMessage(
         FORMAT_MESSAGE_ALLOCATE_BUFFER | 
@@ -132,7 +132,7 @@ void DynLoader::open(const char *libname_P)
 	}
 	handle = WINLOADLIB(LPSTRtoLPWSTR(fulllibname).c_str());
 #elif defined(_WIN32)
-	if (sizeof(void*) != sizeof(HMODULE)) {
+	if constexpr(sizeof(void*) != sizeof(HMODULE)) {
 		errstream << "type of handle in dynload.cpp seems to be wrong" << endl;
 		return;
 	}
@@ -184,8 +184,6 @@ void DynLoader::close()
 				errstream << "not really closing dynamic library because of pthread problem under Linux - contact author for details or check dynload.cpp from pstoedit source code " << libname << endl;
 		// dlclose(handle);
 #else
-		if (libname && verbose)
-				errstream << "closing dynamic library " << libname << endl;
 		dlclose(handle);
 #endif
 
@@ -240,7 +238,12 @@ DynLoader::fptr DynLoader::getSymbol(const char *name, int check) const
 #elif defined(OS_WIN32_WCE)
 	DynLoader::fptr rfptr = ptr_to_fptr(GetProcAddress((HINSTANCE) handle, LPSTRtoLPWSTR(name).c_str()));	//lint !e611 //: Suspicious cast
 #elif defined(_WIN32)
-	DynLoader::fptr rfptr = ptr_to_fptr(GetProcAddress((HINSTANCE) handle, name));	//lint !e611 //: Suspicious cast
+#ifdef _WIN64
+	DynLoader::fptr rfptr = /* ptr_to_fptr */(GetProcAddress((HINSTANCE) handle, name));	//lint !e611 //: Suspicious cast
+#else
+	DynLoader::fptr rfptr = ptr_to_fptr(GetProcAddress((HINSTANCE)handle, name));
+#endif
+
 #else
 #error "system unsupported so far"
 #endif
@@ -286,9 +289,9 @@ public:
 
 static PluginVector LoadedPlugins;
 
-static void loadaPlugin(const char *filename, ostream & errstream, bool verbose)
+static bool loadaPlugin(const char *filename, ostream & errstream, bool verbose)
 {
- 	if (!filename) return;
+ 	if (!filename) return false;
 	if ( verbose )
 		errstream << "loading plugin: " << filename << endl;
 
@@ -299,7 +302,7 @@ static void loadaPlugin(const char *filename, ostream & errstream, bool verbose)
 		errstream << "Problem during opening of pstoedit driver plugin: " << filename <<
 			". This is no problem as long the driver in this library is not needed. Possibly you need to install further libraries and/or extend the LD_LIBRARY_PATH (*nix) or PATH (Windows) environment variables."
 			<< endl;
-		return;
+		return false;
 	} else {
 #if defined (__APPLE__)
 // on MacOS it seems necessary to at least do a dlsym to an existing symbol 
@@ -323,11 +326,12 @@ static void loadaPlugin(const char *filename, ostream & errstream, bool verbose)
 			(getglobalRpFuncPtr) dynloader->getSymbol("getglobalRp");
 		if (dyngetglobalRpFunc == nullptr) {
 			errstream << "could not find getglobalRp " << endl;
-			return;
+			return false;
 		}
-		DescriptionRegister *dynglobalRp = dyngetglobalRpFunc();
+		const DescriptionRegister * const dynglobalRp = dyngetglobalRpFunc();
 		if (dynglobalRp == nullptr) {
 			errstream << " didn't find any registered Drivers " << endl;
+			return false;
 		} else if (dynglobalRp != getglobalRp()) {
 //                      globalRp->explainformats(errstream);
 			getglobalRp()->mergeRegister(errstream, *dynglobalRp, filename);
@@ -339,6 +343,7 @@ static void loadaPlugin(const char *filename, ostream & errstream, bool verbose)
 		// one from the caller
 	}
 //          globalRp->explainformats(errstream);
+  return true;
 }
 
 // for directory search
@@ -368,7 +373,7 @@ static void loadaPlugin(const char *filename, ostream & errstream, bool verbose)
 
 #ifdef DIR_VERSION
 // this is for all *nix like systems
-void loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
+bool loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
 {
 	if (pluginDir) {
 		DIR *dirp;
@@ -395,10 +400,10 @@ void loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
 		if (!dirp) {
 			if (verbose && pluginDir)
 				errstream << "opendir failed on: " << pluginDir << endl;
-			return;
+			return false;
 		}
-
-		while ((direntp = readdir(dirp)) != NIL) {
+        bool plugin_loaded = false;
+		while ((direntp = readdir(dirp)) != nullptr) {
 //      cout <<  direntp->d_name << endl;
 			unsigned int flen = strlen(direntp->d_name);
 			char *expectedpositionofsuffix = direntp->d_name + flen - strlen(suffix);
@@ -417,21 +422,22 @@ void loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
 				strcat_s(fullname, newlen, "/");
 				strcat_s(fullname, newlen, direntp->d_name);
 //          cout <<  direntp->d_name  << " " << fullname << endl;
-				loadaPlugin(fullname, errstream, verbose);
+				plugin_loaded |= loadaPlugin(fullname, errstream, verbose);
 				delete[]fullname;
 			}
 
 		}						// while
 		closedir(dirp);
+		return plugin_loaded;
 	} else {
 		errstream << "Could not load plugins - parameter pluginDir is null " << endl;
-		return;
+		return false;
 	}
 }
 #elif defined(_WIN32)
 
 
-void loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
+bool loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
 {
 	if (pluginDir != nullptr) {
 		char szExePath[1000];
@@ -441,7 +447,8 @@ void loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
 		WIN32_FIND_DATA finddata;
 
 		const char pattern[] = "/*.dll";
-		const size_t size = strlen(pluginDir) + strlen(pattern) + 1; 
+		const size_t pluginDirsize = strlen(pluginDir);
+		const size_t size = pluginDirsize + strlen(pattern) + 1;
 		char *searchpattern = new char[size];
 		strcpy_s(searchpattern, size, pluginDir);
 		strcat_s(searchpattern, size, pattern);
@@ -450,12 +457,13 @@ void loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
 #else
 		HANDLE findHandle = FindFirstFile(searchpattern, &finddata);
 #endif
+		bool plugin_loaded = false;
 		if (findHandle == INVALID_HANDLE_VALUE) {
 			if (verbose)
 				errstream << "Could not open plug-in directory (" << pluginDir
 					<< " or didn't find any plugin there" << endl;
 		} else {
-			BOOL more = true;
+			BOOL more = true;	
 			while (more) {
 				// check for suffix being really .dll because FindFirstFile also matches
 				// files such as e.g. .dllx
@@ -471,7 +479,7 @@ void loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
 				if (STRICMP(&finddata.cFileName[len - 4], ".dll") == 0) {
 #endif
 					// cout << &finddata.cFileName[len -4 ] << endl;
-					const size_t size_2 = strlen(pluginDir) + len + 3; 
+					const size_t size_2 = pluginDirsize + len + 3;
 					char *fullname = new char[size_2];
 					strcpy_s(fullname, size_2, pluginDir);
 					strcat_s(fullname, size_2, "\\");
@@ -491,7 +499,7 @@ void loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
 #endif
 						// avoid loading dll itself again
 						//                 errstream << "loading " << fullname << endl;
-						loadaPlugin(fullname, errstream, verbose);
+						plugin_loaded |= loadaPlugin(fullname, errstream, verbose);
 					} else {
 						//                 errstream << "ignoring myself " << finddata.cFileName << endl;
 					}
@@ -502,9 +510,10 @@ void loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
 			(void) FindClose(findHandle);
 		}
 		delete[]searchpattern;
+		return plugin_loaded;
 	} else {
 		errstream << "Could not load plugins - parameter pluginDir is null " << endl;
-		return;
+		return false;
 	}
 }
 #else
@@ -523,12 +532,13 @@ using namespace std;
 #else
 class ostream;
 #endif
-void loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
+bool loadPlugInDrivers(const char *pluginDir, ostream & errstream, bool verbose)
 {
 	if (verbose) {
 		errstream <<
 			"dummy version of loadPlugInDrivers called since you system doesn't seem to support loading libraries at runtime - If you think that this is not correct, contact the author of this program (wglunz35_AT_pstoedit.net) "
 			<< endl;
 	}
+	return false;
 }								// just a dummy Version
 #endif
