@@ -2,7 +2,7 @@
    dynload.h : This file is part of pstoedit
    declarations for dynamic loading of drivers
 
-   Copyright (C) 1993 - 2020 Wolfgang Glunz, wglunz35_AT_pstoedit.net
+   Copyright (C) 1993 - 2021 Wolfgang Glunz, wglunz35_AT_pstoedit.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -62,8 +62,5 @@ class ostream;
 #endif
 #ifndef LEANDYNLOAD
 // lean - just the dlopen/dlclose stuff
-void loadPlugInDrivers(const char * pluginDir,ostream& errstream,bool verbose);
+bool loadPlugInDrivers(const char * pluginDir,ostream& errstream,bool verbose);
 #endif 
- 
- 
- 
