@@ -2,7 +2,7 @@
    pstoedit.cpp : This file is part of pstoedit
    main control procedure
 
-   Copyright (C) 1993 - 2020 Wolfgang Glunz, wglunz35_AT_pstoedit.net
+   Copyright (C) 1993 - 2021 Wolfgang Glunz, wglunz35_AT_pstoedit.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -258,23 +258,15 @@ void checkheap(const char * tracep, const void * pUserData) {};
 static void loadpstoeditplugins(const char *progname, ostream & errstream, bool verbose)
 {
 	static bool pluginsloaded = false;
-	if (pluginsloaded)
+	if (pluginsloaded) {
 		return;
-	RSString plugindir = getRegistryValue(errstream, "common", "plugindir");
+	}
+	const RSString plugindir = getRegistryValue(errstream, "common", "plugindir");
 	if (plugindir.length()) {
 		loadPlugInDrivers(plugindir.c_str(), errstream, verbose);	// load the driver plugins
-		pluginsloaded = true;
+		//pluginsloaded = true;
 	}
-#ifdef PSTOEDITLIBDIR
-	struct stat s;
-	if (!pluginsloaded &&
-	    !stat(PSTOEDITLIBDIR, &s) &&
-	    S_ISDIR(s.st_mode)) {
-  	  // also try to load drivers from the PSTOEDITLIBDIR
-	  loadPlugInDrivers(PSTOEDITLIBDIR, errstream,verbose);
-	  pluginsloaded = true;
-	}
-#endif
+
 	// If the above failed, also look in the directory where the pstoedit .exe/dll was found
 	if (!pluginsloaded) {
 	  char szExePath[1000];
@@ -284,23 +276,34 @@ static void loadpstoeditplugins(const char *progname, ostream & errstream, bool
 	  char *p = nullptr;
 	  if (r && (p = strrchr(szExePath, directoryDelimiter)) != nullptr) {
 		*p = '\0';
-		loadPlugInDrivers(szExePath, errstream,verbose);
+		pluginsloaded |= loadPlugInDrivers(szExePath, errstream,verbose);
 	  }
 	  // now try also $exepath/../lib/pstoedit
-          if (szExePath[0]) {
-            // it is not an empty string
+      if (szExePath[0]) {
+         // it is not an empty string
 #if COMPILEDFOR64BIT 
 	    strcat_s(szExePath,1000,"/../lib64/pstoedit");
 #else
 	    strcat_s(szExePath,1000,"/../lib/pstoedit");
 #endif
 	    if (!strequal(szExePath, plugindir.c_str())) {
-	      loadPlugInDrivers(szExePath, errstream,verbose);
+	      pluginsloaded |= loadPlugInDrivers(szExePath, errstream,verbose);
 	    }
 	  }
 	}
+#ifdef PSTOEDITLIBDIR
+	if (!pluginsloaded) {
+		struct stat s;
+		if (!pluginsloaded &&
+			!stat(PSTOEDITLIBDIR, &s) &&
+			S_ISDIR(s.st_mode))	{
+			// also try to load drivers from the PSTOEDITLIBDIR
+			loadPlugInDrivers(PSTOEDITLIBDIR, errstream, verbose);
+			pluginsloaded = true;
+		}
+	}
+#endif
 
-	// delete[]plugindir;
 }
 #endif
 
@@ -312,10 +315,9 @@ extern FILE *yyin;				// used by lexer
 static void usage(ostream & outstream, bool forTeX, bool withdetails, bool withcategories = false)
 {
 	if (withcategories) {
-		// PropSheetEnum {g_t, t_t, d_t, a_t, b_t, h_t };
-		for (unsigned int sheet = PsToEditOptions::g_t ; sheet < PsToEditOptions::h_t ; sheet++ ) {
+		for (unsigned int sheet = PsToEditOptions::theOptions().g_t ; sheet < PsToEditOptions::theOptions().h_t ; sheet++ ) {
 	        // if (sheet == PsToEditOptions::a_t) continue; // skip "about"
-			outstream << "\\subsection{"  << PsToEditOptions::propSheetName((PsToEditOptions::PropSheetEnum) sheet) << "}" << endl;
+			outstream << "\\subsection{"  << PsToEditOptions::theOptions().propSheetName(sheet) << "}" << endl;
 			PsToEditOptions::theOptions().showhelp(outstream,forTeX,withdetails,sheet);
 		}
 		outstream << "\\subsection{Input and outfile file arguments}" << endl;
@@ -338,6 +340,11 @@ bool f_useCoutForDiag = false; // default is cout - but some clients may redirec
 extern "C" DLLEXPORT
 void useCoutForDiag(int flag) { f_useCoutForDiag = (flag != 0); }
 
+extern "C" DLLEXPORT 
+ProgramOptions* getProgramOptions() {
+	return &PsToEditOptions::theOptions();
+}
+
 
 extern "C" DLLEXPORT
 	int pstoedit(int argc, const char *const argv[], ostream & errstream,
@@ -400,7 +407,7 @@ extern "C" DLLEXPORT
 #endif
 		errstream << "pstoedit: version " << PACKAGE_VERSION << " / DLL interface " <<
 		drvbaseVersion << " (built: " << __DATE__ << " - " << buildtype << " - " << compversion << ")"
-		" : Copyright (C) 1993 - 2020 Wolfgang Glunz\n";
+		" : Copyright (C) 1993 - 2021 Wolfgang Glunz\n";
 	}
 
 	//  handling of derived parameters
@@ -1560,6 +1567,7 @@ extern "C" DLLEXPORT void setPstoeditOutputFunction(void *cbData, write_callback
 	(void) cbBuffer.set_callback(cbData, cbFunction);
 #if defined(HAVE_STL) || defined(__OS2__)
 	(void)cerr.rdbuf(&cbBuffer);
+//	(void)cout.rdbuf(&cbBuffer);
 #else
 	cerr = &cbBuffer;
 #endif
